/*
 * SSD1306/SSD1315に７セグメントっぽいフォントで表示
 * 
 */
#include <Wire.h>
#include "OLED1306.h"

// SSD1306の初期化コマンド群
static uint8_t OLED_SSD1306_CMD  = 0x80;  // SSD1306の1Byteコマンド
static uint8_t OLED_SSD1306_CMDS = 0x00;  // SSD1306の複数Byteコマンド
static uint8_t OLED_SSD1306_DATA = 0x40;  // SSD1306のデータバイト
static uint8_t OLED_SSD1306_INIT[] = {
  0xC8,         // C0/C8:上下の描画方向
  0xA0,         // A0/A1:左右の描画方向
  0x22,         // 20/21/22/23:アドレッシングモード
  0xA8,0x3F,    // 画面の解像度
  0xD3,0x00,    // 縦方向のオフセット
  0x40,         // 縦方向の描画開始位置
  0xA6,         // 表示方法 - A7にすると画面反転
  0x8D, 0x14    // OLEDパネルの供給電圧と思われる 14hに設定しないとダメらしい
};

// 初期化
void OLED1306::init() {
  // OLED設定コマンド送信
  oledSendCommands(sizeof(OLED_SSD1306_INIT), OLED_SSD1306_INIT) ;  // 初期化
  oledSendCommand(0xAF) ; // 画面表示
  cls() ;
}

void OLED1306::cls() {
  oledFillData(0) ;
}

// OLEDに1バイトコマンドを送信
void OLED1306::oledSendCommand(uint8_t command) {
  Wire.beginTransmission(OLED_SSD1306_ADDR);
  Wire.write(OLED_SSD1306_CMD);
  Wire.write(command);
  Wire.endTransmission();
}

// OLEDに2バイトコマンドを送信
void OLED1306::oledSendCommand2(uint8_t command1,uint8_t command2) {
  Wire.beginTransmission(OLED_SSD1306_ADDR);
  Wire.write(OLED_SSD1306_CMDS);
  Wire.write(command1);
  Wire.write(command2);
  Wire.endTransmission();
}

// OLEDに3バイトコマンドを送信
void OLED1306::oledSendCommand3(uint8_t command1,uint8_t command2,uint8_t command3) {
  Wire.beginTransmission(OLED_SSD1306_ADDR);
  Wire.write(OLED_SSD1306_CMDS);
  Wire.write(command1);
  Wire.write(command2);
  Wire.write(command3);
  Wire.endTransmission();
}

// OLEDに複数のコマンドを送信::Wireクラスの制限により最大32Byte
// 長さのチェックはしてないのでプログラマ責任で上手い事やってちょ
void OLED1306::oledSendCommands(int commandLen,uint8_t commands[]) {
  Wire.beginTransmission(OLED_SSD1306_ADDR);
  Wire.write(OLED_SSD1306_CMDS);
  for (int i = 0; i < commandLen; i++) {
    Wire.write(commands[i]);
  }
   Wire.endTransmission();
}

// 画面を特定のビットバターンで埋める::画面の初期化用
void OLED1306::oledFillData(uint8_t fillData) {
  for (uint8_t m = 0; m < OLED_SSD1306_PAGES; m++) {
    oledSendCommand3(0xB0 | (m & 0x07) , 0x10 , 0x00) ;

    Wire.beginTransmission(OLED_SSD1306_ADDR);
    Wire.write(OLED_SSD1306_DATA);
    for (uint8_t x = 0; x < OLED_SSD1306_WIDTH; x++) {
      if (Wire.write(fillData) == 0) {
        Wire.endTransmission();
        Wire.beginTransmission(OLED_SSD1306_ADDR);
        Wire.write(OLED_SSD1306_DATA);
        Wire.write(fillData);
      }
    }
    Wire.endTransmission();
  }
  oledSendCommand3(0xB0 , 0x10 , 0x00) ;
}

// BITパターン表示
void OLED1306::oledBitMap(int offsetX , int offsetY , int width,int hight, uint8_t bitMap[]) {
  int pos = 0 ;
  for (int m = 0; m < hight; m++) {
    oledSendCommand3(0xB0 | ((m + offsetY) & 0x07) , 0x10 | (offsetX & 0xf0) >> 4 , offsetX & 0x0F) ;
    Wire.beginTransmission(OLED_SSD1306_ADDR);
    Wire.write(OLED_SSD1306_DATA);
    for (int x = 0; x < width; x++) {
      int8_t byteData = bitMap[pos] ;
      if (Wire.write(byteData) == 0) {
        Wire.endTransmission();
        Wire.beginTransmission(OLED_SSD1306_ADDR);
        Wire.write(OLED_SSD1306_DATA);
        Wire.write(byteData);
      }
      pos++ ;
    }
    Wire.endTransmission();
  }
  oledSendCommand3(0xB0 , 0x10 , 0x00) ;
}
