#include "camera_pins.h"
#include "esp_camera.h"

#include <Wire.h>
#include "OLED1306.h"

// ========== OLED ==========
OLED1306 oled ;
uint8_t frameBuffWB[240*176] ; // FRAMESIZE_HQVGA 240x176 : 2値画像用
uint8_t oledBuff[OLED_SSD1306_WIDTH * OLED_SSD1306_PAGES]; // SSD1306表示用

// ========== CAMERA ==========
camera_fb_t* fb    = NULL;
uint8_t* out_jpg   = NULL;
size_t out_jpg_len = 0;

static camera_config_t camera_config = {
    .pin_pwdn     = PWDN_GPIO_NUM,
    .pin_reset    = RESET_GPIO_NUM,
    .pin_xclk     = XCLK_GPIO_NUM,
    .pin_sscb_sda = SIOD_GPIO_NUM,
    .pin_sscb_scl = SIOC_GPIO_NUM,
    .pin_d7       = Y9_GPIO_NUM,
    .pin_d6       = Y8_GPIO_NUM,
    .pin_d5       = Y7_GPIO_NUM,
    .pin_d4       = Y6_GPIO_NUM,
    .pin_d3       = Y5_GPIO_NUM,
    .pin_d2       = Y4_GPIO_NUM,
    .pin_d1       = Y3_GPIO_NUM,
    .pin_d0       = Y2_GPIO_NUM,

    .pin_vsync = VSYNC_GPIO_NUM,
    .pin_href  = HREF_GPIO_NUM,
    .pin_pclk  = PCLK_GPIO_NUM,

    .xclk_freq_hz = 20000000,
    .ledc_timer   = LEDC_TIMER_0,
    .ledc_channel = LEDC_CHANNEL_0,

    .pixel_format  = PIXFORMAT_GRAYSCALE,
    .frame_size    = FRAMESIZE_HQVGA,
    .jpeg_quality  = 0,
    .fb_count      = 2,
    .fb_location   = CAMERA_FB_IN_PSRAM,
    .grab_mode     = CAMERA_GRAB_LATEST,
    .sccb_i2c_port = 0,
};

/**
 * カメラ画像読込み
 */
void ScanImage( ) {
    fb = esp_camera_fb_get();
    if (fb) {
        Serial.print("fb.len=") ;
        Serial.print(fb->len) ;
        Serial.print("   fb.width=") ;
        Serial.print(fb->width) ;
        Serial.print("   fb.height=") ;
        Serial.println(fb->height) ;
        drowBitMapBW(0 , 0 , fb->width, fb->height, fb->buf) ;
        // ----------
        esp_camera_fb_return(fb);
        fb = NULL;
    }
}

/**
 * DispImageBW()
 * 
 * カメラ画像を２値化
 * SSD1306のフレーム形式に変換
 * SSD1306に書き込み
 * 
 */
void drowBitMapBW(int offsetX , int offsetY , int width, int hight, uint8_t bitMap[]) {
    // ----- 2値化
    uint16_t bwBuff[5] ;
    int pos = 0 ;
    int addPos = 0 ;
    int bwAve = 0 ;
    int bwAveN = 0 ;
    int rowN = 0;
    for (int row = 2 ; row < hight-2 ; row ++) {
        memset(bwBuff,0,sizeof(bwBuff)) ;
        pos = row * width ;
        for (int col = 2 ; col < width-2 ; col ++ ) {
          bwBuff[addPos] = bitMap[pos-width*2] ;
          bwBuff[addPos] += bitMap[pos-width] ;
          bwBuff[addPos] += bitMap[pos] ;
          bwBuff[addPos] += bitMap[pos+width] ;
          bwBuff[addPos] += bitMap[pos+width*2] ;
          bwAve = (bwBuff[0] + bwBuff[1] + bwBuff[2] + bwBuff[3] + bwBuff[4]) / 25 ;
          frameBuffWB[pos] = (bitMap[pos] > bwAve) ? 0xFF : 0x00;
          addPos = (addPos + 1) % 5 ;
          bwBuff[addPos] = 0 ;
          pos ++ ;
        }
    }

    // ----- SSD1306表示用バッファに変換
    int offX = (width - OLED_SSD1306_WIDTH) / 2 ;
    int offY = (hight - OLED_SSD1306_HIGHT) / 2 ;

    memset(oledBuff,0,sizeof(oledBuff)) ;
    uint8_t bitMask = 0x01 ;
    for (int y=0;y<OLED_SSD1306_HIGHT;y++) {
      for (int x=0;x<OLED_SSD1306_WIDTH;x++) {
          if (frameBuffWB[(y+offX)+(x+offY)*width] == 0xFF) {
            oledBuff[x+(y/8)*OLED_SSD1306_WIDTH] |=  bitMask ;
          }
      }
      bitMask <<= 1 ;
      if (bitMask == 0) { bitMask = 0x01 ; }
    }

    oled.oledBitMap(0,0,OLED_SSD1306_WIDTH,OLED_SSD1306_PAGES,oledBuff) ;
}

void setup() {
    Serial.begin(115200);

    // ----- CAMERA INITIAL -----
    pinMode(POWER_GPIO_NUM, OUTPUT);
    digitalWrite(POWER_GPIO_NUM, LOW);
    esp_err_t err = esp_camera_init(&camera_config);
    if (err != ESP_OK) {
        Serial.println("Camera Init Fail");
        delay(1000);
        esp_restart();
    } else {
        Serial.println("Camera Init Success");
    }

    // -----  SSD1306 INITIAL -----
    Wire.begin(39,38); // SDA,SCL
    oled.init() ;
}

// the loop function runs over and over again forever
void loop() {
    ScanImage( ) ;
}
