/*
 * SSD1331にイメージデータを表示
 * 
 */
#include "OLED1331.h"
#include <SPI.h>
                        // 直結 / ノーマル
#define TFT_MOSI    6   // 6 / 6 Master In Slave Out
#define TFT_CLK     5   // 5 / 7 Serial Clock

#define TFT_CS     -1   // Slave Select(Chip select)
#define TFT_DC      8   // 8 / 5 SSD1306 Data/Command
#define TFT_RESET    7   // 7 / 8 SSD1306 Reset

uint8_t SPIBuf[256] ;   // SPI転送用バッファ

// --- SSD1331 COMMADN ---
static uint8_t oledInitCommands[] = { // SSD1331の初期化コマンド群
  0xAE,         // DISPLAYOFF
  0xA0 , 0x72,  // SETREMAP 0x72 / 0x76 : RGB Color
// ▼ 設定を変更してみる場合の設定値
//  0xA0 , B00110010 ,  // ノーマル 256色
//  0xA0 , B01110010 ,  // ノーマル 64色 Format1
//  0xA0 , B10110010 ,  // ノーマル 64色 Format2
//  0xA0 , B01100010 ,  // 縦方向ミラー反転 64色 Format1
//  0xA0 , B01110000 ,  // 横方向ミラー反転 64色 Format1
//  0xA0 , B01100000 ,  // 上下反転　      64色 Format1
//  0xA0 , B01100011 ,  // 左90度回転表示　64色 Format1 // ※90度回転は何故か正常に動作しない
//  0xA0 , B01110001 ,  // 右90度回転表示　64色 Format1
  0xA1 , 0x00 , // STARTLINE
  0xA2 , 0x00 , // DISPLAYOFFSET
  0xA4 ,        // NORMALDISPLAY
  0xB0 , 0x0B , // POWERMODE
  0x87 , 0x01 , // MASTERCURRENT : 0x06
  0x81 , 0x91 , // CONTRAST RED  : 0x91
  0x82 , 0x50 , // CONTRAST GREEN: 0x50
  0x83 , 0x7D , // CONTRAST BLUR : 0x7D
  0xAF ,        // DISPLAYON
};

uint8_t drowComandClearWindow[] = {0x25 , 0 , 0 , 95, 63 }; // ClearWindow Command

uint8_t drowComandLineR[] = {0x21 , 0, 0, 95, 63, 0x3E, 0x00, 0x00 }; // DrowLine Command
uint8_t drowComandLineG[] = {0x21 , 0, 63, 95, 0, 0x00, 0x3E, 0x00 }; // DrowLine Command
uint8_t drowComandLineB[] = {0x21 , 95, 63, 0, 0, 0x00, 0x00, 0x3E }; // DrowLine Command
uint8_t drowComandLineBL[] = {0x21, 95, 0, 0, 63, 0x00, 0x00, 0x00 }; // DrowLine Command

uint8_t drowComandRect1[] = {0x22 , 0, 0, 95, 63, 0x00, 0x3E, 0x00, 0x3E, 0x3E, 0x3E }; // DrowRect Command
uint8_t drowComandRect2[] = {0x22 , 0, 0, 47, 63, 0x3E, 0x00, 0x00, 0x00, 0x00, 0x00 }; // DrowRect Command

uint8_t drowComandDimWindow1[] = {0x24 , 0, 32, 95, 63 }; // DimWindow Command
uint8_t drowComandDimWindow2[] = {0x24 , 48, 0, 95, 63 }; // DimWindow Command

uint8_t drowComandCopyWindow[] = {0x23 , 0,  0, 47,63, 48, 0}; // CopyWindow Command

uint8_t drowComandScrooll[] = {0x27 , 2, 32, 31, 0, 2 }; // Scroll Command
uint8_t drowComandScrool2[] = {0x27 , 0, 0, 64, 2, 2 }; // Scroll Command

// ===== SPI WRITE =====
SPISettings settings = SPISettings(4000000, MSBFIRST, SPI_MODE0);
void OLED1331::init( ) {
  SPI.begin(TFT_CLK,-1,TFT_MOSI);  // SPI.begin(SCK, MISO, MOSI, SS);
  SPI.beginTransaction(settings);

  // -----  ST3375 INITIAL -----
  pinMode(TFT_DC, OUTPUT);
  pinMode(TFT_RESET, OUTPUT);

  // --- Reset SSD1306
  pinMode(TFT_RESET, OUTPUT);
  digitalWrite(TFT_RESET, HIGH);
  delay(1);                       // VDD goes high at start, pause for 1 ms
  digitalWrite(TFT_RESET, LOW);  // Bring reset low
  delay(10);                      // Wait 10 ms
  digitalWrite(TFT_RESET, HIGH); // Bring out of reset

  // --- SSD1306 初期化
  // OLED設定コマンド送信
  oledSendCommands(sizeof(oledInitCommands), oledInitCommands) ;  // 初期化
  oledSendCommand(0xA4) ; // 描画ON
  oledSendCommand(0xAF) ; // 画面表示

  // ----- CLEAR WINDOW -----
  memcpy(SPIBuf, drowComandClearWindow, sizeof(drowComandClearWindow)) ;
  oledSendCommands(sizeof(drowComandClearWindow), SPIBuf) ;  // ClearWindow Command
  delayMicroseconds(500);
}

// OLEDに1バイトコマンドを送信
void OLED1331::oledSendCommand(uint8_t command) {
#if TFT_CS != -1
  digitalWrite(TFT_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
#if TFT_CS != -1
  digitalWrite(TFT_CS, HIGH);  // SSD1306解放
#endif
}

// OLEDに2バイトコマンドを送信
void OLED1331::oledSendCommand2(uint8_t command1, uint8_t command2) {
#if TFT_CS != -1
  digitalWrite(TFT_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
#if TFT_CS != -1
  digitalWrite(TFT_CS, HIGH);  // SSD1306解放
#endif
}

// OLEDに3バイトコマンドを送信
void OLED1331::oledSendCommand3(uint8_t command1, uint8_t command2, uint8_t command3) {
#if TFT_CS != -1
  digitalWrite(TFT_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command1);
  SPI.transfer(command2);
  SPI.transfer(command3);
#if TFT_CS != -1
  digitalWrite(TFT_CS, HIGH);  // SSD1306解放
#endif
}

// OLEDに複数のコマンドを送信
void OLED1331::oledSendCommands(int commandLen, uint8_t commands[]) {
#if TFT_CS != -1
  digitalWrite(TFT_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(commands, commandLen);
#if TFT_CS != -1
  digitalWrite(TFT_CS, HIGH);  // SSD1306解放
#endif
}

void OLED1331::drowBitMap(int offsetX , int offsetY , int width, int hight, uint8_t bitMap[]) {
  int dispWide = (width > OLED_SSD1331_WIDTH) ? OLED_SSD1331_WIDTH : width ;
  int dispHight= (hight > OLED_SSD1331_HIGHT) ? OLED_SSD1331_HIGHT : hight ;
  uint8_t endX = offsetX + dispWide - 1;
  uint8_t endY = offsetY + dispHight - 1;
  oledSendCommand3(0x15, offsetX, endX) ;
  oledSendCommand3(0x75, offsetY, endY) ;
  delayMicroseconds(500);

  int pos = 0 ;
  offsetX = (dispWide - OLED_SSD1331_WIDTH) / 2 ;
  offsetX = (offsetX < 0) ? 0 : offsetX ;
  offsetY = (dispHight - OLED_SSD1331_HIGHT) / 2 ;
  offsetY = (offsetY < 0) ? 0 : offsetY ;
#if TFT_CS != -1
  digitalWrite(TFT_CS, LOW); // SSD1306セレクト
#endif
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int row = 0 ; row < dispHight ; row ++) {
      pos = (row+offsetY) * width * 2 + offsetX * 2 ;
//    for (int col = 0 ; col < dispWide * 2 ; col ++ ) {
//      SPIBuf[col] = bitMap[pos];
//      pos ++ ;
//    }
    memcpy(SPIBuf,&bitMap[pos],dispWide * 2) ;
    SPI.transfer(SPIBuf, dispWide * 2);
  }
  digitalWrite(TFT_DC, LOW); // Command mode
#if TFT_CS != -1
  digitalWrite(TFT_CS, HIGH);  // SSD1306解放
#endif
}

void OLED1331::cls( ) {
    // ----- CLEAR WINDOW -----
    memcpy(SPIBuf, drowComandClearWindow, sizeof(drowComandClearWindow)) ;
    oledSendCommands(sizeof(drowComandClearWindow), SPIBuf) ;  // ClearWindow Command
    delayMicroseconds(500);
}
