#include <SPI.h>
#include "OLED1331.h"

#include "camera_pins.h"
#include "esp_camera.h"
#include "OLED1331.h"

OLED1331 oled ;

// ========== CAMERA ==========
uint8_t CAMBuff[96*96*2] ;   // 転送用バッファ
camera_fb_t* fb    = NULL;
uint8_t* out_jpg   = NULL;
size_t out_jpg_len = 0;

static camera_config_t camera_config = {
    .pin_pwdn     = PWDN_GPIO_NUM,
    .pin_reset    = RESET_GPIO_NUM,
    .pin_xclk     = XCLK_GPIO_NUM,
    .pin_sscb_sda = SIOD_GPIO_NUM,
    .pin_sscb_scl = SIOC_GPIO_NUM,
    .pin_d7       = Y9_GPIO_NUM,
    .pin_d6       = Y8_GPIO_NUM,
    .pin_d5       = Y7_GPIO_NUM,
    .pin_d4       = Y6_GPIO_NUM,
    .pin_d3       = Y5_GPIO_NUM,
    .pin_d2       = Y4_GPIO_NUM,
    .pin_d1       = Y3_GPIO_NUM,
    .pin_d0       = Y2_GPIO_NUM,

    .pin_vsync = VSYNC_GPIO_NUM,
    .pin_href  = HREF_GPIO_NUM,
    .pin_pclk  = PCLK_GPIO_NUM,

    .xclk_freq_hz = 20000000,
    .ledc_timer   = LEDC_TIMER_0,
    .ledc_channel = LEDC_CHANNEL_0,

    .pixel_format  = PIXFORMAT_RGB565,
    .frame_size    = FRAMESIZE_96X96,
    .jpeg_quality  = 0,
    .fb_count      = 2,
    .fb_location   = CAMERA_FB_IN_PSRAM,
    .grab_mode     = CAMERA_GRAB_LATEST,
    .sccb_i2c_port = 0,
};

void ScanImage( ) {
    fb = esp_camera_fb_get();
    if (fb) {
        int width = fb->width ;
        int height = fb->height ;
        // --- Rotate Buff ---
        for (int y=0;y<height;y++) {
          uint16_t *posS = (uint16_t *)&fb->buf[y * width * 2] ;
          uint16_t *posD = (uint16_t *)&CAMBuff[y * 2] ;
          for (int x=0;x<width;x++) {
            *posD = *posS ;
            posS++ ;
            posD += width ;
          }
        }
        // ----------
        esp_camera_fb_return(fb);
        fb = NULL;
        if (out_jpg) {
            free(out_jpg);
            out_jpg     = NULL;
            out_jpg_len = 0;
        }
        oled.drowBitMap(0 , 0 , height, width, CAMBuff) ;
    }
}

void setup() {
    Serial.begin(115200);

    // ----- CAMERA INITIAL -----
    pinMode(POWER_GPIO_NUM, OUTPUT);
    digitalWrite(POWER_GPIO_NUM, LOW);

    esp_err_t err = esp_camera_init(&camera_config);
    if (err != ESP_OK) {
        Serial.println("Camera Init Fail");
        delay(1000);
        esp_restart();
    } else {
        Serial.println("Camera Init Success");
    }

    // -----  SSD1331 INITIAL -----
    oled.init( ) ;
}

// the loop function runs over and over again forever
void loop() {
  ScanImage( ) ;
}
