#include "camera_pins.h"
#include "esp_camera.h"

#include <SPI.h>
#include "image2.h"
#include "font8x16.h"
                      // AtomS3/S3R
#define TFT_BL    -1  // -1:-1 BACK LIGHT
#define TFT_CS    -1  // -1:-1 Data/Command
#define TFT_MOSI   6  // 19:6 Master In Slave Out
#define TFT_CLK    7  // 23:7 Serial Clock
#define TFT_DC     5  // 22:5 Data/Command
#define TFT_RST    8  // 33:8 RESET

#if TFT_CS != -1
#define CSWRITE(lev) digitalWrite(TFT_CS,lev)
#else
#define CSWRITE(lev)
#endif

#define TFT_WIDTH 160
#define TFT_HIGH  128
uint8_t SPIBuf[TFT_WIDTH*2] ;   // SPI転送用バッファ
uint8_t fontColorF[2] ; // TEXT FOR GROND COLOR
uint8_t fontColorB[2] ; // TEXT BACK GROND COLOR

char str[64] ;

SPISettings settings = SPISettings(32000000, MSBFIRST, SPI_MODE0);

// ========== CAMERA ==========
camera_fb_t* fb    = NULL;
uint8_t* out_jpg   = NULL;
size_t out_jpg_len = 0;

static camera_config_t camera_config = {
    .pin_pwdn     = PWDN_GPIO_NUM,
    .pin_reset    = RESET_GPIO_NUM,
    .pin_xclk     = XCLK_GPIO_NUM,
    .pin_sscb_sda = SIOD_GPIO_NUM,
    .pin_sscb_scl = SIOC_GPIO_NUM,
    .pin_d7       = Y9_GPIO_NUM,
    .pin_d6       = Y8_GPIO_NUM,
    .pin_d5       = Y7_GPIO_NUM,
    .pin_d4       = Y6_GPIO_NUM,
    .pin_d3       = Y5_GPIO_NUM,
    .pin_d2       = Y4_GPIO_NUM,
    .pin_d1       = Y3_GPIO_NUM,
    .pin_d0       = Y2_GPIO_NUM,

    .pin_vsync = VSYNC_GPIO_NUM,
    .pin_href  = HREF_GPIO_NUM,
    .pin_pclk  = PCLK_GPIO_NUM,

    .xclk_freq_hz = 20000000,
    .ledc_timer   = LEDC_TIMER_0,
    .ledc_channel = LEDC_CHANNEL_0,

    .pixel_format  = PIXFORMAT_GRAYSCALE,
    .frame_size    = FRAMESIZE_QQVGA,
    .jpeg_quality  = 0,
    .fb_count      = 2,
    .fb_location   = CAMERA_FB_IN_PSRAM,
    .grab_mode     = CAMERA_GRAB_LATEST,
    .sccb_i2c_port = 0,
};

// ========== TFT ==========
// TFTにコマンドを送信
void tftSendCommand(uint8_t command) {
  CSWRITE(LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  CSWRITE(HIGH);  //
}

// TFTにコマンド+1バイトデータを送信
void tftSendCommand1(uint8_t command, uint8_t data1) {
  CSWRITE(LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  CSWRITE(HIGH);  //
}

// TFTにコマンド+2バイトデータを送信
void tftSendCommand2(uint8_t command, uint8_t data1, uint8_t data2) {
  CSWRITE(LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  CSWRITE(HIGH);  // 
}

// TFTにコマンド+3バイトデータを送信
void tftSendCommand3(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3) {
  CSWRITE(LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  CSWRITE(HIGH);  //
}

// TFTにコマンド+4バイトデータを送信
void tftSendCommand4(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3, uint8_t data4) {
  CSWRITE(LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  SPI.transfer(data4);
  CSWRITE(HIGH);  //
}

// 画面クリア
void cls(bool rot) {
  if (rot) {
    tftSendCommand4(0x2A,0,0,0,131) ; // Colmun Address
    tftSendCommand4(0x2B,0,0,0,161) ; // Row Address
  } else {
    tftSendCommand4(0x2A,0,0,0,160) ; // Colmun Address
    tftSendCommand4(0x2B,0,0,0,131) ; // Row Address
  }

  CSWRITE(LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int i=0;i<160;i++) {
    for (int j=0;j<131;j++) {
      SPI.transfer(B00000000);
      SPI.transfer(B00000000);
    }
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  CSWRITE(HIGH);  //
}

// 塗りつぶし
void fill(int xs,int ys , int xe,int ye , uint8_t c1 , uint8_t c2) {
  tftSendCommand4(0x2A,0,xs,0,xe) ; // Colmun Address
  tftSendCommand4(0x2B,0,ys,0,ye) ; // Row Address

  CSWRITE(LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int i=0;i<xe-xs+1;i++) {
    for (int j=0;j<ye-ys+1;j++) {
      uint8_t dw = c1 ;
      SPI.transfer(dw);
      dw = c2 ;
      SPI.transfer(dw);
    }
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  CSWRITE(HIGH);  //
}

// BITマップ表示
void drowBitMap(int offsetX , int offsetY , int width, int hight, uint8_t bitMap[]) {
  uint8_t endX = offsetX + width - 1;
  uint8_t endY = offsetY + hight - 1;
  tftSendCommand4(0x2A,0,offsetX,0,endX) ; // Colmun Address
  tftSendCommand4(0x2B,0,offsetY,0,endY) ; // Row Address
  delayMicroseconds(500);

  int pos = 0 ;
  CSWRITE(LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int row = 0 ; row < hight ; row ++) {
    // 表示データ転送
    // Arduinoだとメモリが足りなくて１画面分を一気に転送する事ができない
    // なので、1Lineずつ送信している
    for (int col = 0 ; col < width * 2 ; col ++ ) {
      SPIBuf[col] = bitMap[pos];
      //      SPIBuf[col] = 0xF8 ;
      pos ++ ;
    }
    SPI.transfer(SPIBuf, width * 2);
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  CSWRITE(HIGH);  //
}

// BITマップ表示
void drowBitMapBW(int offsetX , int offsetY , int width, int hight, uint8_t bitMap[]) {
  uint8_t endX = offsetX + width - 1 ;
  uint8_t endY = offsetY + hight - 1 - 2;
  tftSendCommand4(0x2A,0,offsetX,0,endX) ; // Colmun Address
  tftSendCommand4(0x2B,0,offsetY,0,endY) ; // Row Address
  delayMicroseconds(500);

  CSWRITE(LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int row = 1 ; row < hight-1 ; row ++) {
    // 表示データ転送
    // Arduinoだとメモリが足りなくて１画面分を一気に転送する事ができない
    // なので、1Lineずつ送信している
    uint16_t bwBuff[3] ;
    bwBuff[0] = 0 ; bwBuff[1] = 0 ; bwBuff[2] = 0 ;
    int pos = row * width ;
    int addPos = 0 ;
    int bwAve = 0 ;
    for (int col = 0 ; col < width-2 ; col ++ ) {
      bwBuff[addPos] = bitMap[pos-width] ;
      bwBuff[addPos] += bitMap[pos] ;
      bwBuff[addPos] += bitMap[pos+width] ;
      bwAve = (bwBuff[0] + bwBuff[1] + bwBuff[2]) / 9 ;

      SPIBuf[col*2]   = (bitMap[pos] > bwAve) ? 0xFF : 0x00;
      SPIBuf[col*2+1] = (bitMap[pos] > bwAve) ? 0xFF : 0x00;

      addPos = (addPos + 1) % 3 ;
      pos ++ ;
    }
    SPI.transfer(SPIBuf, width * 2);
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  CSWRITE(HIGH);  //
}

void DrowFont(uint8_t code , uint8_t x, uint8_t y) {
  int pos = (code - 0x20) * 16 ;
  memset(SPIBuf,0,8*16*2) ;
  for (int i=0;i<16;i++) {
    uint8_t data = ssd1306xled_font8x16[pos + i];
    uint8_t bitMast = 0x01 ; 
    for (int j=0;j<8;j++) {
      int fpos = 0 ;
      if (i<8) {
        fpos = (j*8+i)*2 ;
      } else {
        fpos = ((j+8)*8+(i-8))*2 ;
      }
      if ((data & bitMast) == 0) {
        SPIBuf[fpos] = fontColorB[0] ;
        SPIBuf[fpos+1] = fontColorB[1] ;
      } else {
        SPIBuf[fpos] = fontColorF[0] ;
        SPIBuf[fpos+1] = fontColorF[1] ;
      }
      bitMast <<= 1 ;
    }
  }

  uint8_t endX = x + 7 ;
  uint8_t endY = y + 15;

  tftSendCommand4(0x2A,0,x,0,endX) ; // Colmun Address
  tftSendCommand4(0x2B,0,y,0,endY) ; // Row Address
  CSWRITE(LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  SPI.transfer(SPIBuf, 8*16*2);
  digitalWrite(TFT_DC, LOW); // Command mode
  CSWRITE(HIGH);  // 
  delayMicroseconds(500);
}

void DrowStr(char str[] , uint8_t x , uint8_t y) {
  for (int i=0;str[i] != 0 ; i++) {
      DrowFont(str[i],x,y) ;
      x += 8 ;
      if (x+8>=96) {
        x = 0 ;
        y += 16 ;
        if (y + 16 >= 64) {
          y = 0 ; 
        }
      }
  }
}

// ========== CAMERA ==========
void ScanImage( ) {
    fb = esp_camera_fb_get();
    if (fb) {
        Serial.print("fb.len=") ;
        Serial.print(fb->len) ;
        Serial.print("   fb.width=") ;
        Serial.print(fb->width) ;
        Serial.print("   fb.height=") ;
        Serial.println(fb->height) ;
        // ----------
//      frame2jpg(fb, 255, &out_jpg, &out_jpg_len);
        drowBitMapBW(0 , 0 , fb->width, fb->height, fb->buf) ;

        // ----------
        esp_camera_fb_return(fb);
        fb = NULL;
        if (out_jpg) {
            free(out_jpg);
            out_jpg     = NULL;
            out_jpg_len = 0;
        }
    }
}


void setup() {
    Serial.begin(115200);
    pinMode(POWER_GPIO_NUM, OUTPUT);
    digitalWrite(POWER_GPIO_NUM, LOW);

    // ----- CAMERA INITIAL -----
    esp_err_t err = esp_camera_init(&camera_config);
    if (err != ESP_OK) {
        Serial.println("Camera Init Fail");
        delay(1000);
        esp_restart();
    } else {
        Serial.println("Camera Init Success");
    }

    // -----  ST3375 INITIAL -----
    SPI.begin(TFT_CLK,-1,TFT_MOSI);  //SPI.begin(SCK, MISO, MOSI, SS);
    SPI.beginTransaction(settings);

#if TFT_CS != 0
    pinMode(TFT_CS, OUTPUT);
#endif
#if TFT_BL != 0
    pinMode(TFT_BL, OUTPUT);
    // BACK LIGHT ON
    digitalWrite(TFT_BL, HIGH);
#endif
    pinMode(TFT_DC, OUTPUT);
    pinMode(TFT_RST, OUTPUT);

    // --- HARD Ware Reset
    digitalWrite(TFT_RST, HIGH);
    delay(500);                  // VDD goes high at start, pause for 500 ms
    digitalWrite(TFT_RST, LOW);  // Bring reset low
    delay(100);                  // Wait 100 ms
    digitalWrite(TFT_RST, HIGH); // Bring out of reset
    delay(500);                  // Wait 500 ms, more then 120 ms
    // --- SOFT Ware Reset
    tftSendCommand(0x01) ;       // SOFTWARE RESET
    delay(50);

    // --- Initial Comands
    bool r = false ;
    tftSendCommand(0x28) ;            // Display OFF : 0x29 - Display ON
    delay(500);

    tftSendCommand(0x11) ;            // Sleep Out
    delay(500);
    tftSendCommand1(0x3A,05) ;        // 16Bit Pixel Mode
    delay(10);
    tftSendCommand1(0x36,B10100000) ; r = false; // MX MY MV ML RGB MH x x:横向き１
//  tftSendCommand1(0x36,B01100000) ; r = false;  // MX MY MV ML RGB MH x x：横向き２
//  tftSendCommand1(0x36,B00000000) ; r = true;  // MX MY MV ML RGB MH x x：縦向き１
//  tftSendCommand1(0x36,B11000000) ; r = true;  // MX MY MV ML RGB MH x x：縦向き２(多分これがスタンダード)
    tftSendCommand2(0xB6,0x15,0x02) ; // Display settings #5
    tftSendCommand1(0xB4,0x00) ;      // Display inversion control

    tftSendCommand(0x13) ;            // NomalDisplayMode
    tftSendCommand(0x20) ;            // Display Inversion Off

    // ---
    cls(r) ;
    delay(500);
    tftSendCommand(0x29) ;            // Display ON : 0x28 - Display OFF
    delay(500);

    fontColorB[0] = 0x00 ; fontColorB[1] = 0x00 ; // TEXT BACK GROND COLOR
    fontColorF[0] = 0xFF ; fontColorF[1] = 0xFF ; // TEXT FOR GROND COLOR - WHITE
}

// the loop function runs over and over again forever
void loop() {
//  sprintf(str,"PARAM:");
//  DrowStr("PARAM:",0,112) ;
  ScanImage( ) ;
}
